#!/bin/sh
# ATTENTION: This script implies a POSIX shell in order to work on busybox without bash.
set -e #< Exit on any error.

print_help()
{
    cat \
<<EOF
The tool for editing .conf files with "name=value" lines.

Usage:

$0 <config_file> <key>
 Print the value. Do nothing if the key is missing.

$0 <config_file> <key> -d|--delete
 Delete the value. Do nothing if the key is missing.

$0 <config_file> <key> <value>
 Set the value, replacing if the key exists. Create the file if it does not exist.

$0 <config_file> -k|--keys
 Print all the keys from the file, separated with spaces.

NOTE: Keys and values starting with '-' may be interpreted as command-line options.

NOTE: In case of most errors with the .conf file, a zero (success) exit status is returned, and
    nothing is printed to stderr and stdout.
EOF
}

fatal_error()
{
    echo >&2 "config_helper.sh ERROR: $1"
    echo >&2 "Run with '-h' or '--help' for more information."
    exit 1
}

main()
{
    if [ $# = 0 ] || [ "$1" = "-h" ] || [ "$1" = "--help" ]; then
        print_help
        exit 0
    fi

    config_file="$1"
    key="$2"
    value="$3"

    if [ -z "${config_file}" ]; then
        fatal_error "The config file is not specified."
    fi
    if [ -z "${key}" ]; then
        fatal_error "The key is not specified."
    fi

    if [ "${key}" = "-k" ] || [ "${key}" = "--keys" ]; then
        # Print all keys.
        grep -E "^\\s*[^[#=]+\\s*=" "${config_file}" | sed "s/\\s*\\(.*\\)\\s*=.*/\\1/"
    elif [ -z "${value}" ]; then
        # Read the value. Keep trailing spaces (if any) - removing them is a hard task.
        if [ ! -f "${config_file}" ]; then
            exit 0
        fi
        # NOTE: `sed "1q"` is used instead of `head -n1` because some Busyboxes don't have `head`.
        grep -E "^\\s*${key}\\s*=" "${config_file}" | sed "1q" | sed "s/.*=\\s*\\(.*\\)/\\1/"
    elif [ "${value}" = "-d" ] || [ "${value}" = "--delete" ]; then
        # Delete the value.
        if [ ! -f "${config_file}" ]; then
            exit 0
        fi
        sed -i "/^\\s*${key}\\s*=/d" "${config_file}"
    else
        if [ ! -f "${config_file}" ]; then
            # Create a new file.
            echo "[General]" >"${config_file}"
            echo "${key}=${value}" >>"${config_file}"
        elif grep -qE "^\\s*${key}\\s*=" "${config_file}"; then
            # Escape sed special characters ("&", "/", "\") in the value.
            escaped_value="$(echo "${value}" | sed 's/[\\/&]/\\&/g')"
            # Replace the value, trimming the key leading and trailing spaces, if any.
            sed -i "s/^\\s*${key}\\s*=.*/${key}=${escaped_value}/" "${config_file}"
        else
            # Insert the setting to the end.
            echo "${key}=${value}" >>"${config_file}"
        fi
    fi
}

main "$@"
