#!/bin/sh
# ATTENTION: This script implies a POSIX shell in order to work on busybox without bash.
set -e #< Exit on any error.

print_help()
{
    cat \
<<EOF
The tool for upgrading or creating the Server configuration file <mediaserver.conf>.

First, if <added_keys.txt> is absent, it is generated as an empty set of keys.

Then, if <mediaserver.conf> is absent, it is copied from <template_mediaserver.conf> if it exists,
all its keys are added to <added_keys.txt>, and no further actions are performed - this takes place
on a clean VMS installation.

Then, if <template_mediaserver.conf> is absent, no further actions are performed - this takes place
when the platform does not have any settings to be written to the config.

Then, the <mediaserver.conf> upgrade is performed by adding those settings from the <template.conf>
that are missing in <added_keys.txt>. The keys of such settings are then added to this file to
avoid being added again during the next VMS installation. If a setting already exists in
<mediaserver.conf>, its value is not changed.

Usage:
$0 \\
    <mediaserver.conf> \\
    <template.conf> \\
    <added_keys.txt> \\
    <./config_helper.sh>
EOF
}

fatal_error()
{
    echo >&2 "Upgrading mediaserver.conf: ERROR: $1"
    echo >&2 "Run with '-h' or '--help' for more information."
    exit 1
}

log() # $*
{
    echo "Upgrading mediaserver.conf: $*"
}

# [in] config_helper_script
config_helper() # $@
{
    "${config_helper_script}" "$@"
}

# Set the exit status to 0 (success) if the key is in the list, to 1 (failure) otherwise.
# list - Whitespace-separated list of items which cannot contain whitespace.
is_key_in_list() # key list
{
    key="$1"
    list="$2"

    for item in ${list}; do
        if [ "${item}" = "${key}" ]; then
            return 0 #< Success.
        fi
    done

    return 1 #< Failure.
}

# Add the setting from the template .conf to mediaserver.conf, replacing the old value, if any.
# [in] config_file
# [in] template_config_file
# [in] config_helper_script
add_template_value_to_config() # key
{
    key="$1"

    value="$(config_helper "${template_config_file}" "${key}")"
    config_helper "${config_file}" "${key}" "${value}"
    log "Added: ${key}=${value}"
}

# [in] config_file The file must exist.
# [in] template_config_file The file must exist.
# [in] added_keys_file The file can be absent.
# [in] config_helper_script
upgrade_existing_config_from_template()
{
    if [ -f "${added_keys_file}" ]; then
        added_keys="$(cat "${added_keys_file}")"
    else
        added_keys=""
    fi

    template_config_keys="$(config_helper "${template_config_file}" --keys)"
    config_keys="$(config_helper "${config_file}" --keys)"

    added_some_values=0
    for template_key in ${template_config_keys}; do
        is_key_in_list "${template_key}" "${added_keys}" && continue
        if is_key_in_list "${template_key}" "${config_keys}"; then
            log "Keeping the already existing value for ${template_key}"
        else
            add_template_value_to_config "${template_key}"
            added_some_values=1
        fi
        echo "${template_key}" >>"${added_keys_file}"
    done

    if [ "${added_some_values}" = 0 ]; then
        log "Not performed - already up to date."
    fi
}

main()
{
    if [ $# = 0 ] || [ "$1" = "-h" ] || [ "$1" = "--help" ]; then
        print_help
        exit 0
    fi

    config_file="$1" #< The file can be absent.
    template_config_file="$2" #< The file can be absent.
    added_keys_file="$3" #< The file can be absent.
    config_helper_script="$4"

    if [ -z "${config_helper_script}" ]; then
        fatal_error "<config_helper.sh> is not specified."
    fi
    if [ ! -f "${config_helper_script}" ]; then
        fatal_error "File not found: \"${config_helper_script}\"."
    fi

    # Generate empty added_keys_file, if absent.
    if [ ! -f "${added_keys_file}" ]; then
        echo "# The keys of the settings that must not be added to mediaserver.conf on upgrade." \
            >"${added_keys_file}"
    fi

    if [ ! -f "${config_file}" ]; then
        log "Not performed - missing the old \"${config_file}\"."
        if [ -f "${template_config_file}" ]; then
            cp "${template_config_file}" "${config_file}"
            log "Created \"${config_file}\" as a copy of \"${template_config_file}\"."
            config_helper "${template_config_file}" --keys >>"${added_keys_file}"
        fi
        return 0
    fi

    if [ ! -f "${template_config_file}" ]; then
        log "Not intended for this platform and VMS version."
        return 0
    fi

    upgrade_existing_config_from_template
}

main "$@"
